description = 'A minimalistic alternative to PyTorch Lightning (no batteries included)'

version_info = (0, 0, 1, 1)

__version__ = '.'.join(map(str, version_info))

__license__ = 'MIT'
__author__ = 'Vladimir Starostin'
__email__ = 'vladimir.starostin@uni-tuebingen.de'

classifiers = [
    'Development Status :: 3 - Alpha',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
]
